==> web_frontend_binect-0.0.55.gem: app/models/nukleus/document_decorator.rb <==
# This configuration will be added by the Rails-Application
# Decorator to add some special features to a document

api_config = Rails.application.config.binect_config
$binect_api = Binectrest::BinectApi.new( { login: api_config[:login], password: api_config[:password], api_wsdl_url: api_config[:api_wsdl_url] } )

Nukleus::Document.class_eval do
    # The document can be associated with a user (if logged in)
    # or with a session.
    field  :user_id,         :type => String,                                           :index => true
    field  :session_id,      :type => String,                                           :index => true
    field  :is_appointed,    :type => NoBrainer::Boolean,       :default => false
    field  :is_payed,        :type => NoBrainer::Boolean,       :default => false
    field  :is_sent,         :type => NoBrainer::Boolean,       :default => false
    field  :zoom_level,      :type => Float,                    :default => 0.0
    field  :x_offset,        :type => Float,                    :default => 0.0
    field  :y_offset,        :type => Float,                    :default => 0.0
    field  :binect_order_id, :type => String,                   :default => ""
    field  :price,           :type => Float,                    :default => 0.0 # In Euro with taxes
    field  :page_count,      :type => Integer,                  :default => 0
    field  :is_color,        :type => NoBrainer::Boolean,       :default => false
    field  :is_simplex,      :type => NoBrainer::Boolean,       :default => true
    field  :is_international,:type => NoBrainer::Boolean,       :default => false
    field  :has_error,       :type => NoBrainer::Boolean,       :default => false
    field  :error_text,      :type => String,                   :default => ""

    after_validation :update_print_params

    def validate
        request_ocr_data
    end

    # Tell binect about user changes
    def update_print_params
        $binect_api.set_shipping_options(self.binect_order_id, self.is_simplex, self.is_color, self.is_international ) if self.binect_order_id.present?
    end

    def self.all_by_user_id(user_id, is_appointed = nil, is_sent = false)
        return [] if user_id.nil?
        if is_appointed.nil?
            Nukleus::Document.where(user_id: user_id, is_sent: is_sent, document_version: 1)
        else
            Nukleus::Document.where(user_id: user_id, is_appointed: is_appointed, is_sent: is_sent, document_version: 1)
        end
    end

    def self.all_by_session_id(session_id, is_appointed = nil, is_sent = false)
        return [] if session_id.nil?
        if is_appointed.nil?
            Nukleus::Document.where(session_id: session_id, is_sent: is_sent, document_version: 1)
        else
            Nukleus::Document.where(session_id: session_id, is_appointed: is_appointed, is_sent: is_sent, document_version: 1)
        end
    end

    def print_settings
        "#{self.is_color ? "color" : "sw"}, #{self.is_simplex ? "simplex" : "duplex"}, #{self.is_international ? "int" : "nat"}"
    end

    private

    def request_ocr_data
        config = Rails.application.config.services[:OCR]
        ocr_project_path = config[:ocr_project_path]

        local_temp_file = self.document_file.download
        resp = ocr_service_connection.put(ocr_project_path) do  |req|
            req.headers['Content-Type'] = self.document_file_content_type
            req.headers['Content-Length'] = local_temp_file.size.to_s
            req.params["customer"] = "Kofax"
            req.body = Faraday::UploadIO.new(local_temp_file.path, self.document_file_content_type)
        end
        # Delete local temp file. Otherwise we will mess up our local file system
        local_temp_file.delete

        Rails.logger.info "OCR-Response status: #{resp.status}"
        # We always want to trigger the next job. Thus, we have to save some meta-data,
        # even if we don't have them.
        hash_to_add = nil
        if resp.status == 200
            if resp.present? && resp.body.present?
                hash_to_add = { ocr_data: resp.body }
            end
        else
            # Error: Save error response as meta-data
            Rails.logger.error "OCR-Response body: #{resp.body}"
            hash_to_add = { ocr_error: resp.body }
        end
        unless hash_to_add.nil?
            self.meta_data = hash_to_add
        end
        self.save
    end

    def ocr_service_connection
        config = Rails.application.config.services[:OCR]
        ocr_service = config[:ocr_service]
        Faraday.new(:url => ocr_service) do |f|
            f.request  :multipart
            f.response :logger         # log requests to STDOUT
            f.adapter  Faraday.default_adapter  # make requests with Net::HTTP
            f.use FaradayMiddleware::ParseJson, content_type: 'application/json'
        end
    end

    # Called by the after_delete handler in Nukleus::Document
    def remove_all_versions
        # We don't want to remove all versions. Thus, we overwrite the default implementation
        # But we want to remove the document in the binect api.
        if binect_order_id.present?
            $binect_api.delete_order(binect_order_id)
        end
    end
end

==> web_frontend_binect-0.0.69.gem: app/models/nukleus/document_decorator.rb <==
# This configuration will be added by the Rails-Application
# Decorator to add some special features to a document

api_config = Rails.application.config.binect_config
$binect_api = Binectrest::BinectApi.new( { login: api_config[:login], password: api_config[:password], api_wsdl_url: api_config[:api_wsdl_url] } )


# This feelds requires a full documentation recreation by the API.
Nukleus::Document::FIELDS_THAT_NEEDS_RECREATION = ["document_version", "is_color", "is_simplex", "x_offset", "y_offset", "zoom_level"]

Nukleus::Document.class_eval do
    # The document can be associated with a user (if logged in)
    # or with a session.
    field  :user_id,         :type => String,                                           :index => true
    field  :session_id,      :type => String,                                           :index => true
    field  :is_appointed,    :type => NoBrainer::Boolean,       :default => false
    field  :is_payed,        :type => NoBrainer::Boolean,       :default => false
    field  :is_sent,         :type => NoBrainer::Boolean,       :default => false
    field  :zoom_level,      :type => Float,                    :default => 0.0
    field  :x_offset,        :type => Float,                    :default => 0.0
    field  :y_offset,        :type => Float,                    :default => 0.0
    field  :binect_order_id, :type => String,                   :default => ""
    field  :price,           :type => Float,                    :default => 0.0 # In Euro with taxes
    field  :page_count,      :type => Integer,                  :default => 0
    field  :is_color,        :type => NoBrainer::Boolean,       :default => false
    field  :is_simplex,      :type => NoBrainer::Boolean,       :default => true
    field  :is_international,:type => NoBrainer::Boolean,       :default => false
    field  :has_error,       :type => NoBrainer::Boolean,       :default => false
    field  :error_text,      :type => String,                   :default => ""

    after_validation :update_print_params

    def validate
        request_ocr_data
    end

    # Tell binect about user changes
    def update_print_params
        $binect_api.set_shipping_options(self.binect_order_id, self.is_simplex, self.is_color, self.is_international ) if self.binect_order_id.present?
    end

    def self.all_by_user_id(user_id, is_appointed = nil, is_sent = false)
        return [] if user_id.nil?
        if is_appointed.nil?
            Nukleus::Document.where(user_id: user_id, is_sent: is_sent, document_version: 1)
        else
            Nukleus::Document.where(user_id: user_id, is_appointed: is_appointed, is_sent: is_sent, document_version: 1)
        end
    end

    def self.all_by_session_id(session_id, is_appointed = nil, is_sent = false)
        return [] if session_id.nil?
        if is_appointed.nil?
            Nukleus::Document.where(session_id: session_id, is_sent: is_sent, document_version: 1)
        else
            Nukleus::Document.where(session_id: session_id, is_appointed: is_appointed, is_sent: is_sent, document_version: 1)
        end
    end

    def print_settings
        "#{self.is_color ? "color" : "sw"}, #{self.is_simplex ? "simplex" : "duplex"}, #{self.is_international ? "int" : "nat"}"
    end

    private

    def request_ocr_data
        config = Rails.application.config.services[:OCR]
        ocr_project_path = config[:ocr_project_path]

        local_temp_file = self.document_file.download
        resp = ocr_service_connection.put(ocr_project_path) do  |req|
            req.headers['Content-Type'] = self.document_file_content_type
            req.headers['Content-Length'] = local_temp_file.size.to_s
            req.params["customer"] = "Kofax"
            req.body = Faraday::UploadIO.new(local_temp_file.path, self.document_file_content_type)
        end
        # Delete local temp file. Otherwise we will mess up our local file system
        local_temp_file.delete

        Rails.logger.info "OCR-Response status: #{resp.status}"
        # We always want to trigger the next job. Thus, we have to save some meta-data,
        # even if we don't have them.
        hash_to_add = nil
        if resp.status == 200
            if resp.present? && resp.body.present?
                hash_to_add = { ocr_data: resp.body }
            end
        else
            # Error: Save error response as meta-data
            Rails.logger.error "OCR-Response body: #{resp.body}"
            hash_to_add = { ocr_error: resp.body }
        end
        unless hash_to_add.nil?
            self.meta_data = hash_to_add
        end
        self.save
    end

    def ocr_service_connection
        config = Rails.application.config.services[:OCR]
        ocr_service = config[:ocr_service]
        Faraday.new(:url => ocr_service) do |f|
            f.request  :multipart
            f.response :logger         # log requests to STDOUT
            f.adapter  Faraday.default_adapter  # make requests with Net::HTTP
            f.use FaradayMiddleware::ParseJson, content_type: 'application/json'
        end
    end

    # Called by the after_delete handler in Nukleus::Document
    def remove_all_versions
        # We don't want to remove all versions. Thus, we overwrite the default implementation
        # But we want to remove the document in the binect api if the first version was removed (thus: everything was removed)
        # TODO: Enable this again if we know why this causes strange problems
        # if binect_order_id.present? && self.document_version == 0
        #     $binect_api.delete_order(binect_order_id)
        # end
    end
end
